/*
 * Davis Vantage VUE
 * 2016-01-16 Kouichi Hirabayashi
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <signal.h>
#include <fcntl.h>
#include <math.h>
#include <time.h>
#include <stdarg.h>
#include <termios.h>
#include <arpa/inet.h>
#include "davis.h"

int serOpen(char *, int);
int serClose(int);
void error(char *, ...);
void Usage();
int receiver(int);
int wakeup(int);
int setTime(int);
int getTime(int);
int download(int, int, int, int, int, int);
int lamp(int, char *);
int getTime(int);
int firmware(int);
int station(int fd);
int rxcheck(int fd);
int rxtest(int fd);
int setPosition(int fd, int, char **);
int setTimezone(int, int, char **);
int eewr(int fd, int adr, int var);
int eebr(int fd, int adr, int var);
int newsetup(int fd);
int loop(int);
int getResponse(int, int, int);
int getResponse2(int, int);
int getSettings(int);
int ckCrc(char *, int);
int convert(char *);
int setCrc(char *, int);
short sVal(char *);
int dump(char *, int);
int test(int);
int Clear(int, int, char **);
int hilow(int);
void tztable(void);
void dumpx(char *, int);

#if defined(RS)
#if defined(__FreeBSD__)
#define PORT	"/dev/cuau0"	// DAVIS WeatherLink SER
elif defined(_linux__)
#define PORT	"/dev/ttyB0"	// DAVIS WeatherLink SER
#else
#error	USB PORT undefined
#endif
#elif defined(USB)
#if defined(__FreeBSD__)
#define PORT	"/dev/cuaU0"	// DAVIS WeatherLink USB Data Logger
#elif defined(_linux__)
#define PORT	"/dev/ttyUSB0"	// DAVIS WeatherLink USB Data Logger
#else
#error	USB PORT undefined
#endif
#elif defined(IP)	// DAVIS WeatherLink IP
#define HOST	"192.168.3.240"
#define PORT	22222
#elif defined(NP)	// OXA Device Server + DAVIS WeatherLink SER
#define HOST	"192.168.3.40"  // NPort5450#3 19200,8,1,No RS-232
#define PORT	4003
//#define HOST	"192.168.3.200"  // NPort DE311
//#define PORT	4001
#else
#error no device defined.
#endif

//#define EUC	// Use EUC-JP Kanji Code


#define TIMEOUT  10

//#define sV(s)	(((*((s) + 1) << 8) & 0xff00) | (*(s) & 0xff))
#define sV(s)	(*(short *)(s))

struct data {
  double ba;	// Barometer (hPa)
  double ti;	// Inside Temperature (C)
  double hi;	// Inside Humidity (%)
  double to;	// outside Temperature (C)
  double ho;	// Outside Humidity (%)
  double ws;	// Wind Speed (m/s)
  double wa;	// 10 Min Average Wind Speed (m/s)
  double wd;	// Wind Direction (clockwise degree)
  double rr;	// Rain Rate (mm/hour)
  double rd;	// Day Rain
  double rm;	// Month Rain
  double ry;	// Year Rain
  int tb;	// Transmitter Battery Status
  double cb;	// Console Battery Voltage
} data;

int Cflg;	// Clear
int Sflg;	// set latitude, loggitude, elevation
int Tflg;	// set time
int Zflg;	// set Time Zone
int cflg;	// get current data
int dflg;	// download archive recod
int hflg;	// get hilow recored
int fflg;	// get firmware version
int lflg;	// lamp on/off
int rflg;	// get receiver
int sflg;	// get settings
int tflg;	// get time
int vflg;	// get version
int xflg;	// dump data
int zflg;	// show Time Zone Table

int fd;
char *cmd;
char buf[BUFSIZ];

int
main(int argc, char **argv)
{
  char *s;
  int c;

  cmd = argv[0];
  while (--argc > 0 && (*++argv)[0] == '-' && !isdigit((*argv)[1]))
	for (s = argv[0]+1; *s; s++)
		switch (*s) {
		case 'C':
			Cflg++;
			break;
		case 'S':
			Sflg++;
			break;
		case 'T':
			Tflg++;
			break;
		case 'Z':
			Zflg++;
			break;
		case 'c':
			cflg++;
			break;
		case 'd':
			dflg++;
			break;
		case 'f':
			fflg++;
			break;
		case 'h':
			hflg++;
			break;
		case 'l':
			lflg++;
			break;
		case 'r':
			rflg++;
			break;
		case 's':
			sflg++;
			break;
		case 't':
			tflg++;
			break;
		case 'v':
			vflg++;
			break;
		case 'x':
			xflg++;
			break;
		case 'z':
			zflg++;
			break;
		default:
			Usage();
		}
#if defined(USB) || defined(RS)
  if ((fd = serOpen(PORT, 19200)) < 0)
	error("can not open %s", PORT);
#elif defined(IP) || defined(NP)
  if ((fd = serOpen(HOST, PORT)) < 0)
	error("can not open %s", HOST);
#endif
  usleep(200000);
  if (wakeup(fd) < 0)
	error("Connection error (wakeup failure)", (char *)0);
  if (Cflg)
	Clear(fd, argc, argv);
  else if (Sflg)
	setPosition(fd, argc, argv);
  else if (Tflg)
	setTime(fd);
  else if (Zflg)
	setTimezone(fd, argc, argv);
  else if (dflg) {
	int y, m, d, H, M;

	y = m = d = H = M = 0;
	if (argc >= 5) {
		y = atoi(argv[0]);
		m = atoi(argv[1]);
		d = atoi(argv[2]);
		H = atoi(argv[3]);
		M = atoi(argv[4]);
	}
	download(fd, y, m, d, H, M);
  }
  else if (fflg)
	firmware(fd);
  else if (hflg)
	hilow(fd);
  else if (lflg) {
	int onoff = 0;

	if (argc > 0)
		lamp(fd, argv[0]);
	else
		Usage();
  }
  else if (rflg) {
//	hilow(fd);
	receiver(fd);
//	test(fd);
//	rxcheck(fd);
//	rxtest(fd);
//	setPosition(fd, 35.7, 139.6, 41.0);
//	tztable();
  }
  else if (sflg)
	getSettings(fd);
  else if (tflg)
	getTime(fd);
  else if (zflg)
	tztable();
  else if (argc == 3) {
	int y, m, d, H, M;

	y = m = d = H = M = 0;
	y = atoi(argv[0]);
	m = atoi(argv[1]);
	d = atoi(argv[2]);
	download(fd, y, m, d, H, M);
  }
  else if (argc == 5) {
	int y, m, d, H, M;

	y = m = d = H = M = 0;
	if (argc >= 5) {
		y = atoi(argv[0]);
		m = atoi(argv[1]);
		d = atoi(argv[2]);
		H = atoi(argv[3]);
		M = atoi(argv[4]);
	}
	download(fd, y, m, d, H, M);
  }
  else
	loop(fd);

//  ver(fd);
//  loop(fd);
//  getTime(fd);
//  setTime(fd);
//  getCal(fd);
  serClose(fd);
  exit(0);
}

int test(int fd)
{
  int n;

  write(fd, "TEST\n", 5);
  n = getResponse(fd, '\r',2);
//  dump(buf, n);
//printf("%s\n", buf + 2);
  return 0;
}


int
wakeup(int fd)
{
  int cnt, i, j, k, n;
  char *s;

  cnt = 0;
  for (i = 0; i < 3; i++) {
	write(fd, "\n", 1);
	if (getResponse(fd, '\r', 1) > 0)
		break;
  }
  if (i == 3) {
	fprintf(stderr, "wakeup() retry over.\n");
	return -1;
  }
  alarm(0);
  return 0;
}

int
firmware(int fd)
{
  int i;
  char *t;

  write(fd, "NVER\n", 5);
  getResponse(fd, '\r', 3);
  *strchr(buf + 6, '\n') = '\0';
  printf("Firmware: Ver. %s  ", buf + 6);
  write(fd, "VER\n", 4);
  getResponse(fd, '\r', 3);
  printf("%s", buf + 6);

  return 0;
}

int
station(int fd)
{
  static char *st[] = { "Wizard III", "Wizard II", "Monitor", "Parception",
	"Gro Wether", "Energy Enviromonitor", "Health Enviromonitor",
	"?", "?", "?", "?", "?", "?", "?", "?", "?",
	"Vantage Pro, Vantage Pro2", "Vantage Vue" };
  int i, k;

  write(fd, "WRD\022\115\n", 6);
  usleep(500000);
  i = read(fd, buf, 2);
//printf("%d\n", i);
//dump(buf, 2);
  k = buf[1] & 0xff;
  buf[2] = '\0';
  printf("Station: %s (0x%02x)\n", st[k], k);

  return 0;
}

int
receiver(int fd)
{
  int i, k;

  write(fd, "RECEIVERS\n", 10);
  usleep(500000);
  i = read(fd, buf, 8);
  k = buf[6] & 0xff;
  buf[7] = '\0';
  if (xflg) {
	for (i = 0; i < 7; i++)
		printf(" 0x%02x", buf[i]&0xff);
	printf("\n");
  }
  if (k == 0)
	printf("Receiver: -\n");
  else
    for (i = 0; i < 16; i++)
	if (k & (0x01 << i) & 1 != 0)
		printf("Receiver: %d\n", i + 1);
  return 0;
}

int
rxcheck(int fd)
{
  int n;

  write(fd, "RXCHECK\n", 8);
  n = getResponse(fd, '\r',3);
//dump(buf, n);
  printf("%s\n", buf + 6);
  return 0;
}

int
rxtest(int fd)
{
  int n;

  write(fd, "RXTEST\n", 7);
  n = getResponse(fd, '\r',1);
  dump(buf, n);
  return 0;
}

int
setPosition(int fd, int argc, char **argv)
{
  double lat, lng, elv;
  char amd[64];
  int v;

//printf("%d %s %s %s\n", argc, argv[0], argv[1], argv[2]);
  if (argc != 3)
	error("Usage: %s -S latitude longitude elevation", cmd);
  lat = lng = elv = 0.0;
  sscanf(argv[0], "%le", &lat);
  sscanf(argv[1], "%le", &lng);
  sscanf(argv[2], "%le", &elv);
//printf("%g %g %g\n", lat, lng, elv);
  v = (int)floor(lat * 10.0 + 0.5);
  sprintf(cmd, "EEWR 0B %02X %02x\n", v >> 8, v & 0xff);
//printf("%s\n", cmd);
  eewr(fd, 0x0B, v);
//return 0;
  v = (int)floor(lng * 10.0 + 0.5);
  sprintf(cmd, "EEWR 0D %02X %02x\n", v >> 8, v & 0xff);
//printf("%s\n", cmd);
  eewr(fd, 0x0D, v);
  v = (int)floor(elv / 0.3048 + 0.5);	// feet to meter
  sprintf(cmd, "EEWR 0F %02X %02x\n", v >> 8, v & 0xff);
//printf("%s\n", cmd);
  eewr(fd, 0x0F, v);
  if (argc == 4)
  newsetup(fd);
  return 0;
}

int
eewr(int fd, int adr, int var)
{
  char cmd[64];
  int n;

  sprintf(cmd, "EEWR %02X %02X\n", adr & 0xff, var & 0xff);
  write(fd, cmd, strlen(cmd));
  usleep(500000);
  if ((n = getResponse(fd, '\r', 2)) < 0)
	fprintf(stderr, "eewr(): response error.\n");
//printf("n(%d):");
//dump(buf, n);
  sprintf(cmd, "EEWR %02X %02X\n", (adr + 1) & 0xff, (var >> 8) & 0xff);
  write(fd, cmd, strlen(cmd));
  usleep(500000);
  if ((n = getResponse(fd, '\r', 2)) < 0)
	fprintf(stderr, "eewr(): response error.\n");
//printf("n(%d):");
//dump(buf, n);
  return 0;
}

int
eebr(int fd, int adr, int var)
{
  char cmd[64];
  int n;

  sprintf(cmd, "EEWR %02X %02X\n", adr & 0xff, var & 0xff);
  write(fd, cmd, strlen(cmd));
  usleep(500000);
  if ((n = getResponse(fd, '\r', 2)) < 0)
        fprintf(stderr, "eewr(): response error.\n");
//printf("n(%d):");
//dump(buf, n);
  return 0;
}

int
newsetup(int fd)
{
  char c;
  int n;

  write(fd, "NEWSETUP\n", 9);
  usleep(500000);
  if ((n = read(fd, &c, 1)) < 0)
	fprintf(stderr, "newsetup(): read error.\n");
  if (c != '\006')
	fprintf(stderr, "expect ACK (received 0x%02x)\n", c & 0xff);
  return 0;
}

int
loop(int fd)
{
  char *t;
  unsigned char c;
  short s;
  unsigned u;
  double x;
  int r, v, noOutside = 0;
  time_t clock;
  struct tm *tm;

  time(&clock);
  tm = localtime(&clock);
  write(fd, "LOOP 1\n", 7);
  r = getResponse2(fd, 99);
//printf("r(%d)\n", r);
//  if (xflg)
//dumpx(buf, 99);
  ckCrc(buf, 99);
  s = sV(buf + 7);
//printf("s(%d)(0x%04x)\n", s, s);
  v = s;
  data.ba = (double)v * 0.0254 * 1013.0 / 760.0;
//  printf(" %.1f hPa\n", (double)v * 0.0254 * 1013.0 / 760.0);
  s = sV(buf + 9);
//printf("%d\n", s);
  data.ti = ((double)s * 0.1 - 32.0) * 5.0 / 9.0;
//  printf("ⲹ %.1f \n", ((double)s * 0.1 - 32.0) * 5.0 / 9.0);
  c = *(buf + 11);
//printf("%d\n", s);
  v = c;
  data.hi = (double)v;
//  printf("⼾ %d %%\n", v);
  s = sV(buf + 12);
//printf("%d\n", s);
//printf(" %d 0x%4x\n", s, s);
  if (s == 0x7fff) {
	noOutside = 1;
	data.to = 0.0;
	data.ho = 0.0;
	data.ws = 0.0;
	data.wa = 0.0;
	data.wd = 0.0;
	data.rr = 0.0;
	data.rd = 0.0;
	data.rm = 0.0;
	data.ry = 0.0;
  }
  else {
	data.to = ((double)s * 0.1 - 32.0) * 5.0 / 9.0;
//  printf(" %.1f \n", ((double)s * 0.1 - 32.0) * 5.0 / 9.0);
	c = *(buf + 33);
//printf("%d\n", s);
	v = c;
//printf(" %d 0x%2x\n", c, c);
	data.ho = (double)v;
//  printf(" %d %%\n", v);
	c = *(unsigned char *)(buf + 14);
//printf("ִ® %d 0x%2x\n", c, c);
//  v = c;
//  data.ws = (double)c * 1609.344 / 3600.0;
	data.ws = (double)c * 1609.344 / 3600.0;
//  printf("ִ® %.1f m/s\n", (double)u * 1609.344 / 3600.0);
	c = *(unsigned char *)(buf + 15);
//  v = c;
//  data.wa = (double)v * 1609.344 / 3600.0;
	data.wa = (double)c * 1609.344 / 3600.0;
//  printf("ʿ® %.1f m/s\n", (double)u * 1609.344 / 3600.0);
	s = sV(buf + 16);
//printf("ʿ® %d 0x%2x\n", s, s);
	data.wd = (double)s;
//  printf(" %d \n", s);
	s = sV(buf + 41);
//printf(" %d 0x%2x\n", s, s);
	data.rr = (double)s * 0.254;
//  printf("Ψ %.0f mm/hour\n", (double)s * 0.254);
	s = sV(buf + 50);
	data.rd = (double)s * 0.254;
//  printf("ֹ߱ %.0f mm\n", (double)s * 0.254);
	s = sV(buf + 52);
	data.rm = (double)s * 0.254;
//  printf("ֹ߱ %.0f mm\n", (double)s * 0.254);
	s = sV(buf + 54);
	data.ry = (double)s * 0.254;
//  printf("ǯֹ߱ %.0f mm\n", (double)s * 0.254);
  }
  data.tb = buf[86] & 0xff;
  if (data.tb != 0)
	fprintf(stderr, "Transmitter Battery Low (Staus: 0x%02x)\n", data.tb);
//	u = *(short *)(buf + 87);
	u = sVal(buf + 87);
  data.cb = (double)u * 300.0 / 512e2;
  if (data.cb < 0.9 * 3.0)
	fprintf(stderr, "Console Battey Voltage low.\n");

  if (cflg) {
#ifdef EUC
	printf("     %.1f hPa\n", data.ba);
	printf("ⲹ %.1f %%\n", data.ti);
	printf("⼾ %.0f %%\n", data.hi);
#else
	printf("Barometer          %.1f hPa\n", data.ba);
	printf("Inside Temperature   %.1f %%\n", data.ti);
	printf("Inside HUmidity       %.0f %%\n", data.hi);

#endif
	if (!noOutside) {
#ifdef EUC
		printf(" %.1f %%\n", data.to);
		printf(" %.0f %%\n", data.ho);
		printf("ִ® %.1f m/s\n", data.ws);
		printf("ʿ® %.1f m/s\n", data.wa);
		printf(" %.0f \n", data.wd);
		printf("Ψ %.0f mm/hour\n", data.rr);
		printf("ֹ߱ %.0f mm\n", data.rd);
		printf("ֹ߱ %.0f mm\n", data.rm);
		printf("ǯֹ߱ %.0f mm\n", data.ry);
#else
		printf("Outside Temperature   %.1f %%\n", data.to);
		printf("Outside Humidity      %.0f %%\n", data.ho);
		printf("Wind Speed            %.1f m/s\n", data.ws);
		printf("10 min Avg Wind Speed %.1f m/s\n", data.wa);
		printf("Wind Direction         %.0f\n", data.wd);
		printf("Rain Rate       %8.0f mm/hour\n", data.rr);
		printf("Day Rain        %8.0f mm\n", data.rd);
		printf("Month Rain      %8.0f mm\n", data.rm);
		printf("Year Rain       %8.0f mm\n", data.ry);

#endif
	}
	s = sV(buf + 91);
	v = s;
#ifdef EUC
	printf("ν   %02d:%02d\n", v / 100, v % 100);
#else
	printf("Time of Sunrise      %02d:%02d\n", v / 100, v % 100);
#endif
	s = sV(buf + 93);
	v = s;
#ifdef EUC
	printf(" %02d:%02d\n", v / 100, v % 100);
#else
	printf("Time os Sunset       %02d:%02d\n", v / 100, v % 100);
#endif
#ifdef EUC
	printf(" 0x%02x\n", buf[70]);
	printf("̷ 0x%02x\n", buf[71]);
	printf(" 0x%02x\n", *(unsigned short *)(buf + 72));

	printf("Хåƥ 0x%02x\n", buf[86]);
#else
	printf("Inside Alarms:  0x%02x\n", buf[70]);
	printf("Rain Alarms:    0x%02x\n", buf[71]);
	printf("Outside Alarms: 0x%02x\n", *(unsigned short *)(buf + 72));

	printf("Transmitter Battey Status: 0x%02x\n", buf[86]);
#endif
	u = sV(buf + 87);
	x = (double)u * 300.0 / 512e2;
	// UM2 x 3 -> 1.5 V * 3 = 4.5 V
#ifdef EUC
	printf("󥽡ХåƥŰ: %.2f V (%.0f %%)\n", x, x / 0.045);
#else
	printf("Console Battey Voltage: %.2f V (%.0f %%)\n", x, x / 0.045);
#endif
  }
  else {
	printf("%04d:%02d%02d:%02d%02d:", tm->tm_year+1900, tm->tm_mon+1,
		tm->tm_mday, tm->tm_hour, tm->tm_min);
	printf("%.1f:%.1f:%.0f:%.1f:%.0f:%.1f:%.1f:%.0f:%.0f:%.0f:%.0f:%.0f:%x:%.2f\n",
		data.ba, data.ti, data.hi, data.to, data.ho, data.ws, data.wa,
		data.wd, data.rr, data.rd, data.rm, data.ry, data.tb, data.cb);
  }
  return 0;
}

int
getTime(int fd)
{
  int r, y, m, d, H, M, S;

  write(fd, "GETTIME\n", 8);
  r = getResponse2(fd, 8);
//printf("r(%d)\n", r);
//  dump(buf, 8);
  ckCrc(buf, 10);
  S = buf[0];
  M = buf[1];
  H = buf[2];
  d = buf[3];
  m = buf[4];
  y = buf[5] + 1900;
  printf("%04d-%02d-%02d %02d:%02d:%02d\n", y, m, d, H, M, S);
  return 0;
}

int
setTime(int fd)
{
  char *s, str[64];
  time_t clock;
  struct tm *tm;
  int r, y, m, d, H, M, S;

  time(&clock);
  clock++;
  tm = localtime(&clock);
  s = str;
  *s++ = tm->tm_sec;
  *s++ = tm->tm_min;
  *s++ = tm->tm_hour;
  *s++ = tm->tm_mday;
  *s++ = tm->tm_mon + 1;
  *s++ = tm->tm_year;
//dump(buf, 6);
  setCrc(str, 6);
//dump(buf, 8);
//ckCrc(buf, 8);
  write(fd, "SETTIME\n", 8);
  if ((r = getResponse2(fd, 0)) < 0)
	return -1;
  write(fd, str, 8);
  if ((r = getResponse2(fd, 0)) < 0)
	return -1;

  return 0;
}

int
setTimezone(int fd, int argc, char **argv)
{
  int index;

  if (argc != 1)
	error("Usage: %s -Z index", cmd);
  sscanf(argv[0], "%d", &index);
  if (index < 0 || index > 46)
	error("setTimezone() error: index must be between 0 and 46. ", (char *)0);
  eebr(fd, 0x11, index);
  eebr(fd, 0x16, 0);	// GMT_OR_ZONE: use TIME_ZONE
//  newsetup(fd);

  return 0;
}

int
getSettings(int fd)
{
  static char *bu[] = { "0.01 INCHES", "0.1 MM", "0.1 HPA", "0.1 MB" };
  static char *tu[] = { "F", "0.1F", "C", "0.1C" };
  static char *eu[] = { "FEET", "METERS" };
  static char *ru[] = { "INCHES", "MM" };
  static char *wu[] = { "MPH", "M/S", "Km/h", "KNOTS" };
  char *t;
  int i, k, r, v;
  short s;

  station(fd);
  receiver(fd);
  firmware(fd);
  write(fd, "EEBRD 1 42\n", 11);
  r = getResponse2(fd, 44);
//printf("r(%d)\n", r);
//  dump(buf, 44);
  s = sV(buf);
#if 1
  printf("BAR_GAIN %d (0x%02x) ( ѹػ)\n", s, s & 0xffff);
  s = sV(buf + 2);
  printf("BAR_OFFSET %d (0x%04x) ( ѹػ)\n", s, s & 0xffff);
  s = sV(buf + 4);
  printf("BAR_CAL %d (0x%04x) (\"BAR=\" ޥɤѹǽ)\n", s, s & 0xffff);
  s = sV(buf + 6);
  printf("HUM33 %d (0x%04x) ( ѹػ)\n", s, s & 0xffff);
  s = sV(buf + 8);
  printf("HUM80 %d (0x%04x) ( ѹػ)\n", s, s &0xffff);
#endif
  s = sV(buf + 10);
//printf("LATITUDE %d (0x%04x)\n", s, s);
  printf(" %.1f (0x%04x)\n", (double)s * 0.1, s & 0xffff);
  s = sV(buf + 12);
//printf("LONGITUDE %d (0x%04x)\n", s, s);
  printf(" %.1f (0x%04x)\n", (double)s * 0.1, s & 0xffff);
  s = sV(buf + 14);
//printf("ELEVATION %d (0x%04x)\n", s, s);
  printf(" %.0f m (0x%04x)\n", s * 0.3048, s & 0xffff);
  k = buf[16] & 0xff;
  printf("TIME_ZONE %d %s\n", k, tztab[k].name);
  if (buf[17] == 0)
	printf("automatic daylight saving.\n");
  else
	printf("manual daylight saving.\n");
//printf("17 0x%02x\n", buf[17]);
  printf("Daylight saving %s\n", buf[18] ? "on" : "off");
//printf("19 20 0x%02x 0x%02x\n", buf[19], buf[20]);
  k = sV(buf + 19);
  if (k < 0)
	printf("GMT offset %.2f hour\n", (double)k / 100.0);
  else
	printf("GMT offset +%.2f hour\n", (double)k / 100.0);
//printf("21 0x%02x\n", buf[21]);
 printf("GMT_OR_ZONE use the %s value\n", buf[21] ? "GMT_OFFSET" : "TIME_ZONE");
//printf("22 0x%02x\n", buf[22]);
#if 1
  printf("USETX: 0x%02x\n", buf[22] & 0xff);
  printf("RE_TRANSMIT_TX: 0x%02x\n", buf[23]&0xff);
  printf("STATION_LIST:");
  for (i = 0; i < 8; i++) {
//	printf(" 0x%02x 0x%02x", buf[24+2*i] & 0xff, buf[25+ 2*i] & 0xff);
	if (i > 0)
		printf(", ");
	if (buf[24+2*i] == 0x00)
		printf(" ISS - Vantage Vue");
	else if (buf[24+2*i] == 4)
		printf(" Wireless Anemometer Station");
	else if (buf[24+2*i] == 5)
		printf("ISS - Vantage Pro 2");
	else if (buf[24+2*i] == 0xA)
//		printf(" No station - OFF");
		printf(" -");
	else
		printf(" Station Type 0x%02x 0x%02x",
			(buf[24+2*i] & 0xff) << 8, buf[25+ 2*i] & 0xff);
  }
  printf("\n");
#endif
  k = buf[40];
  printf("ñ: %s\n", bu[k & 0x03]);
  printf("ñ: %s\n", tu[(k >> 2) & 0x3]);
  printf("ñ: %s\n", eu[(k >> 4) & 0x1]);
  printf("ñ: %s\n", ru[(k >> 5) & 0x3]);
  printf("®ñ: %s\n", wu[(k >> 6) & 0x3]);
  return 0; 
}

int
getCal(int fd)
{
  int r;

  write(fd, "EEBRD 32 2B\n", 12);
  r = getResponse2(fd, 45);
  dump(buf, 45);

  return 0;
}

int
Clear(int fd, int argc, char **argv)
{
  int i, n;
  char c, cmd[64];

  for (i = 0; i < argc; i++) {
	if (strcmp(argv[i], "log") == 0) {
		write(fd, "CLRLOG\n", 7);
		if ((n = read(fd, &c, 1)) < 1)
			fprintf(stderr, "Clear(): log failed.\n");
	}

  }
  return 0;
}

/*
 * y-m-d H:M ʹߤΥ֥쥳ɤ
 */

int
download(int fd, int y, int m, int d, int H, int M)
{
  char *t, str[64];
  int r, i, j, k, n, p, bn;
  unsigned short dates, times, s;
  time_t clock;
  struct tm *tm;

  if (y == 0) {
	time(&clock);
	tm = localtime(&clock);
	y = tm->tm_year+1900;
	m = tm->tm_mon+1;
	d = tm->tm_mday;
	H = tm->tm_hour;
	M = tm->tm_min;
  }
  dates = (unsigned short)(d + m * 32 + (y - 2000) * 512);
  times = (unsigned short)(H * 100 + M);
//printf("y(%d) m(%d) d(%d) H(%d) M(%D)\n", y, m, d, H, M);
//printf("0x%02x 0x%02x\n", dates, times);
  write(fd, "DMPAFT\n", 7);
  r = getResponse2(fd, 0);
//printf("r(%d)\n", r);
//  dump(buf, 1);
  t = str;
  *(unsigned short*)t = dates;
  t += 2;
  *(unsigned short*)t = times;
  t += 2;
  setCrc(str, 4);
//dump(str, 6);
  write(fd, str, 6);
  r = getResponse2(fd, 6);
//printf("r(%d)\n", r);
//dump(buf, 6);
  p = *(unsigned short*)buf;
//printf("page(%d)\n", p);
  write(fd, "\006", 1);
  usleep(500000);
/*
usleep(500000);
  write(fd, "\033", 1);
*/
  for (j = 0; j < p; j++) {
	sleep(1);
	alarm(TIMEOUT);
	if (read(fd, buf, 1) < 0) {
		alarm(0);
		fprintf(stderr, "read() error.\n");
		perror("first");
		return -1;
	}
	bn = buf[0] & 0xff;	// block sequence number
//printf("Block(%d)\n", bn);
	for (t = buf, i = n = 0; n < 266; ) {
		if ((k = read(fd, t, 266)) < 0) {
			fprintf(stderr, "read() error (%d).\n", k);
			perror("netx");
			alarm(0);
			return -1;
		}
//printf("n(%d)(0x%02x)\n", k, *t & 0xff);
		n += k;
		t += k;
//printf("n(%d) i(%d), k(%d)\n", n, i, k);
		if (266 <= n)
			break;
	}
	alarm(0);
//dumpBlock(bn, buf);
	convert(buf);
//break;
	write(fd, "\006", 1);
  }
usleep(500000);
  write(fd, "\033", 1);
  return 0;
}

int
convert(char *buf)
{
  char *t;
  short s;
  unsigned short u;
  unsigned long l;
  int i, j, k, y, m, d, H, M, v;
  double x;

  for (t = buf, i = 0; i < 5; i++) {
	u = *(unsigned short *)t;
	if (u == 0xffff)
		break;
	y = ((u >> 9) & 0x7f) + 2000;
	m = (u >> 5) & 0x0f;
	d = u & 0x1f;
//printf("y(%d) m(%d) d(%d)\n", y, m, d);
	u = *(unsigned short *)(t + 2);
	H = u / 100;
	M = u % 100;
//printf("H(%d) M(%d)\n", H, M);
	s = *(unsigned short *)(t + 4);
	data.to = ((double)s * 0.1 - 32.0) * 5.0 / 9.0;
//printf(" %.1f \n", data.to);
	s = sV(t + 14);
	v = s;
	data.ba = (double)v * 0.0254 * 1013.0 / 760.0;
//printf(" %.1f hPa\n", data.ba);
	s = sV(t + 20);
        data.ti = ((double)s * 0.1 - 32.0) * 5.0 / 9.0;
//printf("ⲹ %.1f \n", data.ti);
	v = t[22] & 0xff;
	data.hi = (double)v;
//printf("⼾ %.0f\n", data.hi);
	v = t[23] & 0xff;
	data.ho = (double)v;
//printf(" %.0f\n", data.ho);
	v = t[24] & 0xff;
	data.wa = (double)v * 1609.344 / 3600.0;
//printf("ʿ® %.0f m\n", data.wa);
	v = t[25] & 0xff;
	data.ws = (double)v * 1609.344 / 3600.0;
//printf("ǹ® %.0f m\n", data.ws);
	v = t[26] & 0xff;
//printf("Direction of Hi Wind Speed %d\n", v);
	data.wd = t[27] & 0xff;
//printf("Privaring Wind Direction %d\n", data.wd);
	s = sV(t + 10);	// Rainfall
	data.rr = ((double)s * 25.4) * 256.0;
	data.rd = data.rm = data.ry = 0.0;
	data.tb = 0;  data.cb = 0.0;
	printf("%04d:%02d%02d:%02d%02d:", y, m, d, H, M);
	printf("%.1f:%.1f:%.0f:%.1f:%.0f:%.1f:%.1f:%.0f:%.0f:%.0f:%.0f:%.0f:%x:%.2f\n",
		data.ba, data.ti, data.hi, data.to, data.ho, data.ws, data.wa,
	data.wd, data.rr, data.rd, data.rm, data.ry, data.tb, data.cb);

	t += 52;
  }
  return 0;
}

int
dumpBlock(int n, char *s)
{
  int i, j;

  printf("Block %d:\n", n);
  for (i = 0; i < 5; i++) {
	for (j = 0; j < 52; j++)
		printf(" %02x", s[i * 52 + j] & 0xff);
	printf("\n");
  }
  for (i = 0; i <= 6; i++)
	printf(" 0x%02x", *(s + 260 + i) & 0xff);
  printf("\n");
  return 0;
}

int
lamp(int fd, char *onoff)
{
  int val = 0;
  char str[64];

  if (strcmp(onoff, "on") == 0 || strcmp(onoff, "1") == 0)
	val = 1;
  sprintf(str, "LAMPS %d\n", val);
  write(fd, str, strlen(str));
  getResponse(fd, '\r', 2);
  return 0;
}

int
hilow(int fd)
{
  int f, i, n, v;
  int y1, y2, m1, m2, d1, d2;
  int Y1, Y2, M1, M2, D1, D2;
  unsigned short v1, v2, v3;
  char *s;

  write(fd, "HILOWS\n", 7);
  if ((n = read(fd, buf, 1)) < 0)
	error("hilow(): read() error.", (char *)0);
  if (buf[0] != 0x06)
	error("hilow(): received 0x%02x (expect ACK)\n", &buf[0]);
  sleep(2);
  if ((n = read(fd, buf, 438)) < 0)
	error("hilo(): read() error. n(%d)", n);
  ckCrc(buf, 438);
  s = buf;
#ifdef EUC
  printf(":\n");
#else
  printf("Barometer Section:\n");
#endif
  d1 = sV(s);
  d2 = sV(s + 2);
  m1 = sV(s + 4);
  m2 = sV(s + 6);
  y1 = sV(s + 8);
  y2 = sV(s + 10);
  v1 = sV(s + 12);
  v2 = sV(s + 14);

#ifdef EUC
  printf(" ֺ㵤: %6.1f\n", (double)d1 * 0.0254 * 1013.0 / 760.0);
  printf(" ֺǹⵤ: %6.1f\n", (double)d2 * 0.0254 * 1013.0 / 760.0);
  printf(" ֺ㵤: %6.1f\n", (double)m1 * 0.0254 * 1013.0 / 760.0);
  printf(" ֺǹⵤ: %6.1f\n", (double)m2 * 0.0254 * 1013.0 / 760.0);
  printf(" ǯֺ㵤: %6.1f\n", (double)y1 * 0.0254 * 1013.0 / 760.0);
  printf(" ǯֺǹⵤ: %6.1f\n", (double)y2 * 0.0254 * 1013.0 / 760.0);
  printf(" ǹⵤ:  %02d:%02d\n", v2 / 100, v2 % 100);
#else
  printf(" Daily Low Barometer:   %6.1f\n", (double)d1 * 0.0254 * 1013.0 / 760.0);
  printf(" Daily High Barometer:  %6.1f\n", (double)d2 * 0.0254 * 1013.0 / 760.0);
  printf(" Month Low Bar:         %6.1f\n", (double)m1 * 0.0254 * 1013.0 / 760.0);
  printf(" Month High Bar:        %6.1f\n", (double)m2 * 0.0254 * 1013.0 / 760.0);
  printf(" Year Low Bar:          %6.1f\n", (double)y1 * 0.0254 * 1013.0 / 760.0);
  printf(" Year High Bar:         %6.1f\n", (double)y2 * 0.0254 * 1013.0 / 760.0);
  printf(" Time of Day Low Bar:    %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" Time of Day hi Bar:     %02d:%02d\n", v2 / 100, v2 % 100);
#endif

  if ((s[17] & 0xff) == 0xff) goto g1;
  printf("Wind Speed Section:\n");
  d1 = s[16] & 0xff;
  v1 = sV(s + 17);
  m1 = s[19] & 0xff;
  y1 = s[20] & 0xff;
#ifdef EUC
  printf(" ֺǹ®: %6.1f\n", (double)d1 * 1609.344 / 3600.0);
  printf(" ǹ®:  %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" ֺǹ®: %6.1f\n", (double)m1 * 1609.344 / 3600.0);
  printf(" ǯֺǹ®: %6.1f\n", (double)y1 * 1609.344 / 3600.0);
#else
  printf(" Day Hi Wind Speed:     %6.1f\n", (double)d1 * 1609.344 / 3600.0);
  printf(" Time of Hi Speed:       %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" Month Hi Wind Speed:   %6.1f\n", (double)m1 * 1609.344 / 3600.0);
  printf(" Year Hi Wind Speed:    %6.1f\n", (double)y1 * 1609.344 / 3600.0);
#endif
g1:
  printf("Inside Temp Section:\n");
  d1 = sV(s + 21);
  d2 = sV(s + 23);
  v1 = sV(s + 25);
  v2 = sV(s + 27);
  m1 = sV(s + 29);
  m2 = sV(s + 31);
  y1 = sV(s + 33);
  y2 = sV(s + 35);

#ifdef EUC
  printf(" ֲǹⵤ: %6.1f\n", ((double)d1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ֲ㵤: %6.1f\n", ((double)d2 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ֺǹⵤ:  %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" ֺ㵤:  %02d:%02d\n", v2 / 100, v2 % 100);
  printf(" ֲǹⵤ: %6.1f\n", ((double)m1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ֲ㵤: %6.1f\n", ((double)m2 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ǯֲǹⵤ: %6.1f\n", ((double)m1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ǯֲ㵤: %6.1f\n", ((double)m2 * 0.1 - 32.0) * 5.0 / 9.0);

  printf("⼾:\n");
#else
  printf(" Day Hi Inside Temp:    %6.1f\n", ((double)d1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Day Low Inside Temp:   %6.1f\n", ((double)d2 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Time Day Hi in Temp:    %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" Time Day Low in Temp:   %02d:%02d\n", v2 / 100, v2 % 100);
  printf(" Month Hi Inside Temp:  %6.1f\n", ((double)m1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Month Low Inside Temp: %6.1f\n", ((double)m2 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Year Hi Inside Temp:   %6.1f\n", ((double)m1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Year Low Inside Temp:  %6.1f\n", ((double)m2 * 0.1 - 32.0) * 5.0 / 9.0);

  printf("Inside Humidity Section:\n");
#endif

  printf("Inside Humidity Section:\n");
  d1 = s[37] & 0xff;
  d2 = s[38] & 0xff;
  v1 =  sV(s + 39);
  v2 =  sV(s + 41);
  m1 = s[43] & 0xff;
  m2 = s[44] & 0xff;
  y1 = s[45] & 0xff;
  y2 = s[46] & 0xff;
#ifdef EUC
  printf(" ֺǹ⼾:       %2d %%\n", d1);
  printf(" ֺ㼾:       %2d %%\n", d2);
  printf(" ǹ⼾ٻ:      %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" 㼾ٻ:      %02d:%02d\n", v2 / 100, v2 % 100);
  printf(" ֺǹ⼾:       %2d %%\n", m1);
  printf(" ֺ㼾:       %2d %%\n", m2);
  printf(" ǯֺǹ⼾:       %2d %%\n", y1);
  printf(" ǯֺ㼾:       %2d %%\n", y2);
#else
  printf(" Day Hi In Hum:           %2d %%\n", d1);
  printf(" Day Low In Hum:          %2d %%\n", d2);
  printf(" Time Day Hi in Hum:     %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" Time Day Low in Hum:    %02d:%02d\n", v2 / 100, v2 % 100);
  printf(" Month Hi In Hum:         %2d %%\n", m1);
  printf(" Month Low In Hum:        %2d %%\n", m2);
  printf(" Year Hi In Hum:          %2d %%\n", y1);
  printf(" Year Low Inside Temp:    %2d %%\n", y2);
#endif

  if ((s[47] & 0xff) == 0xff) goto g2;
  printf("Outside Temp Section:\n");
  d1 = sV(s + 47);
  d2 = sV(s + 49);
  v1 = sV(s + 51);
  v2 = sV(s + 53);
  m1 = sV(s + 55);
  m2 = sV(s + 57);
  y1 = sV(s + 59);
  y2 = sV(s + 61);

#ifdef EUC
  printf(" ֲǹⵤ: %6.1f\n", ((double)d2 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ֲ㵤: %6.1f\n", ((double)d1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ֺǹⵤ:  %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" ֺ㵤:  %02d:%02d\n", v2 / 100, v2 % 100);
  printf(" ֲǹⵤ: %6.1f\n", ((double)m1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ֲ㵤: %6.1f\n", ((double)m2 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ǯֲǹⵤ: %6.1f\n", ((double)m1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" ǯֲ㵤: %6.1f\n", ((double)m2 * 0.1 - 32.0) * 5.0 / 9.0);
#else
  printf(" Day Hi Outside Temp:   %6.1f\n", ((double)d2 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Day Low Outside Temp:  %6.1f\n", ((double)d1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Time Day Hi in Temp:    %02d:%02d\n", v1 / 100, v1 % 100);
  printf(" Time Day Low in Temp:   %02d:%02d\n", v2 / 100, v2 % 100);
  printf(" Month Hi Out Temp:     %6.1f\n", ((double)m1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Month Low Out Temp:    %6.1f\n", ((double)m2 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Year Hi Out Temp:      %6.1f\n", ((double)m1 * 0.1 - 32.0) * 5.0 / 9.0);
  printf(" Year Low Out Temp:     %6.1f\n", ((double)m2 * 0.1 - 32.0) * 5.0 / 9.0);
#endif
g2:

  return 0;
}

/*
 * cnt: # of byte to receive
 */

int
getResponse2(int fd, int cnt)
{
  int i, j, k, n;
  char *s;

//printf("getResponse2(%d, %d)\n", fd, cnt);
  usleep(500000);
  alarm(TIMEOUT);
  if (read(fd, buf, 1) < 0) {
	alarm(0);
	fprintf(stderr, "read() error.\n");
	return -1;
  }
  alarm(0);
//printf("ACK(%02x)\n", buf[0] & 0xff);
  alarm(TIMEOUT);
  if (buf[0] != 0x06) {
	alarm(0);
	if (buf[0] == 0x21)
		fprintf(stderr, "Invalid command parameter (received NAK).\n");
	else if (buf[0] == 0x18)
		fprintf(stderr, "CRC error (received CAN).\n");
	else
		fprintf(stderr, "Unexpected responce (0x%02x).\n",
			buf[0] & 0xff);
	return -1;
  }
  if (cnt == 0) {
	alarm(0);
	return 0;
  }
//  alarm(TIMEOUT);
  for (s = buf, i = n = 0; n < sizeof(buf); ) {
	if ((k = read(fd, s, cnt)) < 0) {
		fprintf(stderr, "read() error (%d).\n", k);
		perror("");
		alarm(0);
		return -1;
	}
//printf("n(%d)(0x%02x)\n", k, *s & 0xff);
	n += k;
	s += k;
//printf("n(%d) cnt(%d), k(%d)\n", n, cnt, k);
	if (cnt <= n)
		break;
  }
  alarm(0);
  return n;
}

/*
 * del: delimiter, cnt: # of delimitters till return
 */

int
getResponse(int fd, int del, int cnt)
{
  int i, j, k, n;
  char *s;

  usleep(200000);
  alarm(TIMEOUT);
  for (s = buf, i = n = 0; n < sizeof(buf); ) {
	if ((k = read(fd, s, 10)) < 0) {
		alarm(0);
		return -1;
	}
//printf("n(%d)(0x%02x)\n", k, *s & 0xff);
//dump(s, k);
	n += k;
	for (j = 0; j < k; j++)
		if (*s++ == del) {
			if (--cnt <= 0) {
				*s = '\0';
				goto j9;
			}
		}
  }
j9:
  alarm(0);
//  dump(buf, n);
  return n;
}

void
tztable(void)
{
  int i;

  printf("Index GMT_Offset Name\n");
  for (i = 0; tztab[i].index != -1; i++) {
	printf("  %2d    %5d  %s\n",
	tztab[i].index, tztab[i].offset, tztab[i].name);
  }
}

short sVal(char *s)
{
  short v;

//printf("0x%02x 0x%02x\n", *s, *(s + 1));
  v = ((*(s + 1) << 8) & 0xff00) | (*s & 0xff);
//printf("v(0x%04x)\n", v & 0xffff);
  return v;
}

#if 0
serGetc(int fd)
{
  char ch;
  int n;

  alarm(TIMEOUT);
  if ((n = read(fd, &ch, 1)) <= 0)
        return -1;
  alarm(0);
//printf("n(%d)\n", n);
  return ((int)ch) & 0xff;
}
#endif

int
dump(char *s, int n)
{
  static char *ascii[] = {
	"NUL", "SOH", "STX", "ETX", "EOT", "ENQ", "ACK", "BEL",
	" BS", " HT", " LF", " VT", " FF", " CR", " SO", " SI",
	"DLE", "DC1", "DC2", "DC3", "DC4", "NAK", "SYN", "ETB",
	"CAN", " EM", "SUB", "ESC", " FS", " GS", " RS", " US",
	" SP"
  }; 
  int i, c;

  for (i = 0; i < n; i++) {
	c = s[i] & 0xff;
	if (c < '\041')
		printf(" %s", ascii[c]);
	else if (c < '\177')
		printf("  %c ", c);
	else if (c == '\177')
		printf(" DEL");
	else
		printf("  %02x", c);
	if ((i % 16) == 15)
		printf("\n");
  }
  if ((i % 16) != 15)
	printf("\n");
  return 0;
}

void
dumpx(char *s, int n)
{
  int i;

  for (i = 0; i < n; i++) {
	if ((i % 16) == 0)
		printf("%3x", (i / 16) * 16);
	printf(" %02x", buf[i] & 0xff);
	if ((i % 16) == 15)
		printf("\n");
  }
  if ((i % 16) != 15)
	printf("\n");
}


void
error(char *s, ...)
{
  va_list args;

  if (fd > 0)
	serClose(fd);
  va_start(args, s);
  fprintf(stderr, "error: ");
  vfprintf(stderr, s, args);
  fprintf(stderr, "\n");
  va_end(args);

  exit(1);
}

#ifdef EUC
void
Usage()
{
  static char *t[] = {
"ޥ %s - Divis Vantage Vue ",
"ʸ	%s [ץ]",
"ץ",
"  -S   	# ¬",
"  -T			# ",
"  -Z TimeZone		# ॾ",
"  -c			# ͤɽ",
"  -d [yyyy mm dd HH MM]	# ʹߤΥ֥쥳ɤɽ",
"  -f			# ե०Сɽ",
"  -h			# ǹ⡢ͤɽ",
"  -l {on|off}		# 󥽡롦饤Ȥ ON/OFF"
"  -r			#  ID (1 - 8) ɽ",
"  -s			# ͤɽ",
"  -t			# ɤߤ",
"  -x			# ǥХåɽ",
"  -z			# Time Zone table ɽ",
"ץȰʤǵưȡǸͤɽޤ",
"  YYYYMMDD:HHMM:ba:ti:hi:to:ho:ws:wa:wd:rr:rd:rm:ry:tb:cb",
"  ˡ",
"	yyyy ǯ",
"       mmdd ",
"	HHMM ʬ",
"	ba    (hPa)",
"	ti   ⲹ ()",
"	hi   ⼾ (%%)",
"	to     ()",
"	ho    (%%)",
"	ws   ִ® (m/s)",
"	wd    ()",
"	wa   10ʬʿ® (m/s)",
"	rr    (mm/hour)",
"	rd   ֱ (mm/day)",
"	rm   ֱ (mm/month)",
"	rd   ǯֱ (mm/year)",
"-c ץϾ嵭˲äơ顼νʬ䤹ɽ",
"",
"-d ץϻꤵ줿ʹߤΥ֥ǡɤߤɽޤ",
"ꤷʤ֥ǡɤߤޤʤ֥ǡ",
"Ǥֱֶ̤֤˱̤߱ˤʤäƤơֱ̤ǯֱ",
"Ϥޤ",
"-f ץϥե०ΥСɽޤ",
"-l onoffץϥ󥽡ΥХå饤Ȥ ON/OFF ؤޤonoff ",
"0 ʤ OFF1 ʤ ON ˤʤޤ",
"-s ץͤɽޤ",
"-t ץϸ߻ɽޤ",
"-z ץ -Z ץǽʥॾ󡦥ơ֥ɽޤ",
"-S latitude, loggitude, elevation ϴ¬ΰ١١٤ꤷޤ"
"-T ϥԥ塼θ߻ Vantage 󥽡˥åȤޤ",
"-Z index ϴ¬Υॾ򥳥󥽡˥åȤޤ",
  NULL
};
  int i;

  for (i = 0; t[i] != NULL; i++) {
	fprintf(stderr, t[i], cmd);
	fprintf(stderr, "\n");
  }
  exit(1);
}

#else

void
Usage()
{
  static char *t[] = {
"Name %s - Divis Vantage Vue acces",
"Synopsis	%s [options]",
"Options",
"  -S latitude longitude altitude	# Observation location setting",
"  -T			# Time setting",
"  -Z TimeZone		# Time Zone setting",
"  -c			# Show current condition",
"  -d [yyyy mm dd HH MM]	# Display archive records after the specified time",
"  -f			# Show firmware version",
"  -h			# Show highest and lowest values ",
"  -l {on|off}		# Console light ON / OFF"
"  -r			# Display receiving ID (1-8)",
"  -s			# Display the set value",
"  -t			# Read the time",
"  -x			# Show debug information",
"  -z			# Show Time Zone table",
"Decroption",
"When started with no options and arguments, the current value is displayed in",
"the following format.",
"  YYYYMMDD:HHMM:ba:ti:hi:to:ho:ws:wa:wd:rr:rd:rm:ry:tb:cb",
"  Where",
"	yyyy Year",
"       mmdd Month Date",
"	HHMM Hour Minutes",
"	ba   Barometric pressure (hPa)",
"	ti   Rome temperature (C)",
"	hi   Room humidity (%%)",
"	to   Outside air temperature (C)",
"	ho   Outside air humidity (%%)",
"	ws   Wind speed (m/s)",
"	wa   10 min avg wind speed (m/s)",
"	wd   Wind direction (degree)",
"	rr   Rain rate (mm/hour)",
"	rd   Day rain (mm/day)",
"	rm   Month rain (mm/month)",
"	rd   Year rain (mm/year)",
"In addition to the above, the -c option clearly displays alarms and sunrise /",
"sunset. ",
"The -d option reads and displays archived data after the specified date and",
"time. If you do not specify the date and time, all archive data will be read.",
"In the archive data, the daily rainfall is the rainfall in the archive",
"section, and there is no monthly rainfall or annual rainfall. ",
"The -f option displays the firmware version.",
"The -l onoff option toggles the console backlight ON / OFF. If onoff is 0,",
"0it is OFF, and if it is 1, it is ON",
"The -s option displays the setting value.",
"The -t option displays the current time. ",
"The -z option displays a time zone table that can be set with the -Z option.",
"The -S option sets the latitude, longitude and altitude of the observation",
"location."
"The -Z option sets the time zone of the observation location to the console.",
  NULL
};
  int i;

  for (i = 0; t[i] != NULL; i++) {
	fprintf(stderr, t[i], cmd);
	fprintf(stderr, "\n");
  }
  exit(1);
}
#endif


#define MAXRETRY	2	/* max. open restry */

static struct sigaction saalrm, osaalrm;
static void onalrm(int);

static void onalrm(int i)
{
  saalrm.sa_handler = onalrm;
  sigemptyset(&saalrm.sa_mask);
  saalrm.sa_flags = 0;
  sigaction(SIGALRM, &saalrm, &osaalrm);
}

//#include <sys/types.h>

#if defined(IP) || defined(NP)

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

static struct hostent *shost;
static char srv_addr[80];
static struct sockaddr_in sv_addr;

int serOpen(char *host, int port)
{
  int i, sofd;

  for (i = 0; i < MAXRETRY; i++) {
	strcpy(srv_addr, host);	/* IP Address */
	saalrm.sa_handler = onalrm;
	sigemptyset(&saalrm.sa_mask);
	saalrm.sa_flags = 0;
	sigaction(SIGALRM, &saalrm, &osaalrm);
	alarm(TIMEOUT);
	if ((sofd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		error("socket() failed", (char *)0);
	if ((shost = gethostbyname(srv_addr)) == NULL)
		error("gethostbyname() failed", (char *)0);
	memset((char *)&sv_addr, 0, sizeof(sv_addr));
	sv_addr.sin_family = AF_INET;
	sv_addr.sin_port = htons(port);
	memcpy((char *)&sv_addr.sin_addr, (char *)shost->h_addr,
		shost->h_length);
	if (connect(sofd, (struct sockaddr *)&sv_addr, sizeof(sv_addr)) < 0) {
		close(sofd);
		sleep(1);
		continue;
	}
	alarm(0);
	break;
  }
  if (i == MAXRETRY)
	return -1;
  usleep(10000);
  return sofd;
}

int serClose(int fd)
{
  if (shutdown(fd, 2) < 0)
        error("shutdown() failed", (char *)0);
  if (close(fd) < 0)
	error("cose(%d) failed", fd);
  return 0;
}

#elif defined(USB)
#define PORT	"/dev/cuaU0"	// DAVIS WeatherLink USB Data Logger
#elif defined(SER)
//#define PORT	"/dev/cuau0"	// DAVIS WeatherLink SER
#endif

#if defined(USB) || defined(SER)

#include <termios.h>

int serOpen(char *dev, int speed)
{
  struct termios tios;
  int fd;

  /* reopen to enabme c_cc[VMIN] and c_cc[VTIME] */
  if ((fd = open(dev, O_RDWR)) == -1)
        error("can't open %s", dev);
  tios.c_iflag = 0;
  tios.c_oflag = 0;
  tios.c_cflag = CLOCAL|CREAD|CS8|HUPCL;
  tios.c_lflag = 0;
  tios.c_cc[VEOL] = '\r';
  tios.c_cc[VMIN] = 1;
  tios.c_cc[VTIME] = 88;
  if (cfsetospeed(&tios, speed) == -1)
        error("cfsetospeed() failed.", (char *)0);
  if (cfsetispeed(&tios, speed) == -1)
        error("cfsetispeed() failed.", (char *)0);
  if (tcsetattr(fd, TCSANOW, &tios) == -1)
        error("ioctl(TCSANOW) failed", (char *)0);

  return fd;
}

int serClose(int fd)
{
  return close(fd);
}

#endif
